@@ -1,4 +1,4 @@
-/* $Id: spf.c,v 1.33 2011/08/17 01:06:50 manu Exp $ */
+/* $Id: spf.c,v 1.40 2015/10/30 18:22:30 manu Exp $ */
 
 /*
  * Copyright (c) 2004-2007 Emmanuel Dreyfus
@@ -34,7 +34,7 @@
 #ifdef HAVE_SYS_CDEFS_H
 #include <sys/cdefs.h>
 #ifdef __RCSID
-__RCSID("$Id: spf.c,v 1.33 2011/08/17 01:06:50 manu Exp $");
+__RCSID("$Id: spf.c,v 1.40 2015/10/30 18:22:30 manu Exp $");
 #endif
 #endif
 
@@ -109,38 +109,48 @@ spf_check_internal(ad, as, ap, priv)
 	SPF_smtp_from(p, from);
 	p->RES = SPF_policy_main(p);
 
-	if (conf.c_debug)
-		mg_log(LOG_DEBUG, "SPF return code %d", p->RES);
-
 	status = ad ? *(enum spf_status *)ad : MGSPF_PASS;
-	switch (status) {
-	case MGSPF_PASS:
-		result = (p->RES == SPF_PASS);
+	switch (p->RES) {
+	case SPF_PASS:
+		result = (status == MGSPF_PASS);
+		strcpy(priv->priv_spf_result, "pass");
 		break;
-	case MGSPF_FAIL:
-		result = (p->RES == SPF_H_FAIL);
+	case SPF_H_FAIL:
+		result = (status == MGSPF_FAIL);
+		strcpy(priv->priv_spf_result, "fail");
 		break;
-	case MGSPF_SOFTFAIL:
-		result = (p->RES == SPF_S_FAIL);
+	case SPF_S_FAIL:
+		result = (status == MGSPF_SOFTFAIL);
+		strcpy(priv->priv_spf_result, "softfail");
 		break;
-	case MGSPF_NEUTRAL:
-		result = (p->RES == SPF_NEUTRAL);
+	case SPF_NEUTRAL:
+		result = (status == MGSPF_NEUTRAL);
+		strcpy(priv->priv_spf_result, "neutral");
 		break;
-	case MGSPF_UNKNOWN:
-		result = (p->RES == SPF_UNKNOWN || p->RES == SPF_UNMECH);
+	case SPF_UNKNOWN:
+	case SPF_UNMECH:
+		result = (status == MGSPF_UNKNOWN);
+		strcpy(priv->priv_spf_result, "unknown");
 		break;
-	case MGSPF_ERROR:
-		result = (p->RES == SPF_ERROR);
+	case SPF_ERROR:
+		result = (status == MGSPF_ERROR);
+		strcpy(priv->priv_spf_result, "error");
 		break;
-	case MGSPF_NONE:
-		result = (p->RES == SPF_NONE);
+	case SPF_NONE:
+		result = (status == MGSPF_NONE);
+		strcpy(priv->priv_spf_result, "none");
 		break;
 	default:
-		mg_log(LOG_ERR, "Internal error: unexpected spf_status");
+		mg_log(LOG_ERR, "Internal error: unexpected spf result %d",
+			" spf test code %d", p->RES, status);
 		exit(EX_SOFTWARE);
 		break;
 	}
 
+	if (conf.c_debug)
+		mg_log(LOG_DEBUG, "SPF return %s (test code %d, result %d)",
+			priv->priv_spf_result, status, result);
+
 	SPF_close(p);
 
 out1:
@@ -255,36 +265,46 @@ spf_check_internal(ad, as, ap, priv)
 #endif
 
 	status = ad ? *(enum spf_status *)ad : MGSPF_PASS;
-	switch (status) {
-	case MGSPF_PASS:
-		result = (out.result == SPF_RESULT_PASS);
+	switch (out.result) {
+	case SPF_RESULT_PASS:
+		result = (status == MGSPF_PASS);
+		strcpy(priv->priv_spf_result, "pass");
 		break;
-	case MGSPF_FAIL:
-		result = (out.result == SPF_RESULT_FAIL);
+	case SPF_RESULT_FAIL:
+		result = (status == MGSPF_FAIL);
+		strcpy(priv->priv_spf_result, "fail");
 		break;
-	case MGSPF_SOFTFAIL:
-		result = (out.result == SPF_RESULT_SOFTFAIL);
+	case SPF_RESULT_SOFTFAIL:
+		result = (status == MGSPF_SOFTFAIL);
+		strcpy(priv->priv_spf_result, "softfail");
 		break;
-	case MGSPF_NEUTRAL:
-		result = (out.result == SPF_RESULT_NEUTRAL);
+	case SPF_RESULT_NEUTRAL:
+		result = (status == MGSPF_NEUTRAL);
+		strcpy(priv->priv_spf_result, "neutral");
 		break;
-	case MGSPF_UNKNOWN:
-		result = (out.result == SPF_RESULT_UNKNOWN);
+	case SPF_RESULT_UNKNOWN:
+		result = (status == MGSPF_UNKNOWN);
+		strcpy(priv->priv_spf_result, "unknown");
 		break;
-	case MGSPF_ERROR:
-		result = (out.result == SPF_RESULT_ERROR);
+	case SPF_RESULT_ERROR:
+		result = (status == MGSPF_ERROR);
+		strcpy(priv->priv_spf_result, "error");
 		break;
-	case MGSPF_NONE:
-		result = (out.result == SPF_RESULT_NONE);
+	case SPF_RESULT_NONE:
+		result = (status == MGSPF_NONE);
+		strcpy(priv->priv_spf_result, "none");
 		break;
 	default:
-		mg_log(LOG_ERR, "Internal error: unexpected spf_status");
+		mg_log(LOG_ERR,
+			"Internal error: unexpected spf_alt/spf_2_10 result %d",
+			" spf test code %d", out.result, status);
 		exit(EX_SOFTWARE);
 		break;
 	}
 
 	if (conf.c_debug)
-		mg_log(LOG_DEBUG, "SPF return code %d", out.result);
+		mg_log(LOG_DEBUG, "SPF return %s (test code %d, result %d)",
+			priv->priv_spf_result, out.result, result);
 
 	SPF_free_output(&out);
 out3:
@@ -315,6 +335,7 @@ spf_check_internal(ad, as, ap, priv)
 	struct mlfi_priv *priv;
 {
 	struct sockaddr *sa = SA(&priv->priv_addr);
+	char *fqdn;
 	char *helo = priv->priv_helo;
 	char *fromp = priv->priv_from;
 	SPF_server_t *spf_server;
@@ -339,6 +360,11 @@ spf_check_internal(ad, as, ap, priv)
 		goto out2;
 	}
 
+	/* Local machine FQDN */
+	fqdn = smfi_getsymval(priv->priv_ctx, "{j}");
+	if (fqdn != NULL && SPF_server_set_rec_dom(spf_server, fqdn) != 0)
+		mg_log(LOG_WARNING, "SPF_server_set_rec_dom failed");
+
 	/*
 	 * Get the IP address
 	 */
@@ -389,37 +415,59 @@ spf_check_internal(ad, as, ap, priv)
 	res = SPF_response_result(spf_response);
 
 	status = ad ? *(enum spf_status *)ad : MGSPF_PASS;
-	switch (status) {
-	case MGSPF_PASS:
-		result = (res == SPF_RESULT_PASS);
+	switch (res) {
+	case SPF_RESULT_PASS:
+		result = (status == MGSPF_PASS);
+		strcpy(priv->priv_spf_result, "pass");
 		break;
-	case MGSPF_FAIL:
-		result = (res == SPF_RESULT_FAIL);
+	case SPF_RESULT_FAIL:
+		result = (status == MGSPF_FAIL);
+		strcpy(priv->priv_spf_result, "fail");
 		break;
-	case MGSPF_SOFTFAIL:
-		result = (res == SPF_RESULT_SOFTFAIL);
+	case SPF_RESULT_SOFTFAIL:
+		result = (status == MGSPF_SOFTFAIL);
+		strcpy(priv->priv_spf_result, "softfail");
 		break;
-	case MGSPF_NEUTRAL:
-		result = (res == SPF_RESULT_NEUTRAL);
+	case SPF_RESULT_NEUTRAL:
+		result = (status == MGSPF_NEUTRAL);
+		strcpy(priv->priv_spf_result, "neutral");
 		break;
-	case MGSPF_UNKNOWN:
-		result = 
-		    (res == SPF_RESULT_PERMERROR || res == SPF_RESULT_INVALID);
+	case SPF_RESULT_INVALID:
+		result = (status == MGSPF_UNKNOWN);
+		strcpy(priv->priv_spf_result, "unknown");
 		break;
-	case MGSPF_ERROR:
-		result = (res == SPF_RESULT_TEMPERROR);
+	case SPF_RESULT_PERMERROR:
+		result = (status == MGSPF_ERROR);
+		strcpy(priv->priv_spf_result, "permerror");
 		break;
-	case MGSPF_NONE:
-		result = (res == SPF_RESULT_NONE);
+	case SPF_RESULT_TEMPERROR:
+		result = (status == MGSPF_ERROR);
+		strcpy(priv->priv_spf_result, "temperror");
+		break;
+	case SPF_RESULT_NONE:
+		result = (status == MGSPF_NONE);
+		strcpy(priv->priv_spf_result, "none");
 		break;
 	default:
-		mg_log(LOG_ERR, "Internal error: unexpected spf_status");
+		mg_log(LOG_ERR, "Internal error: unexpected spf2 result %d",
+			" spf test code %d", res, status);
 		exit(EX_SOFTWARE);
 		break;
 	}
 
 	if (conf.c_debug)
-		mg_log(LOG_DEBUG, "SPF return code %d", res);
+		mg_log(LOG_DEBUG, "SPF return %s (test code %d, result %d)",
+			priv->priv_spf_result, res, result);
+
+	if (priv->priv_spf_header != NULL)
+		free(priv->priv_spf_header);
+
+	
+	if (spf_response->received_spf_value)
+		priv->priv_spf_header =
+		    strdup(spf_response->received_spf_value);
+	else
+		priv->priv_spf_header = NULL;
 
 	SPF_response_free(spf_response);
 out3:
@@ -480,7 +528,8 @@ acl_print_spf(ad, buf, len)
 		tmpstr = "self";
 		break;
 	default:
-		mg_log(LOG_ERR, "Internal error: unexpected spf_status");
+		mg_log(LOG_ERR, "Internal error: unexpected spf_status %d",
+			status);
 		exit(EX_SOFTWARE);
 		break;
 	}
@@ -524,7 +573,8 @@ spf_check(ad, as, ap, priv)
 		break;
 	}
 
-	mg_log(LOG_ERR, "Internal error: unexpected spf_status");
+	mg_log(LOG_ERR, "Internal error: unexpected spf_status %d",
+		status);
 	exit(EX_SOFTWARE);
 
 	return 0;
@@ -534,9 +584,9 @@ spf_check(ad, as, ap, priv)
  * Check if the SPF record is wide open: the simpliest
  * way of doing it is to check whether our own IP 
  * validates the record.
- * That is a problem for Postfix installations, as the
- * local IP is not available. For now this clause is 
- * not available on Postfix.
+ * That is a problem for Postfix and CommSuite installations,
+ * as the local IP is not available ({if_addr} milter macro).
+ * For now such MTAs must define "localaddr" in config.
  */
 static int
 spf_check_self(ad, as, ap, priv)
@@ -563,9 +613,14 @@ spf_check_self(ad, as, ap, priv)
 			     sizeof(struct sockaddr_in6));
 		break;
 #endif /* AF_INET6 */
-	default:
-#ifndef USE_POSTFIX
+	default:	/* localaddr unspecified, try fallback to {if_addr} */
 		ip = local_ipstr(priv);
+		if (strcmp(ip, "0.0.0.0") == 0) {
+			mg_log(LOG_ERR,
+				"spf self used without localaddr specified "
+				"and no {if_addr} macro is available");
+			return 0;
+		}
 		if (strncmp(ip, "IPv6:", strlen("IPv6:")) == 0) {
 #ifdef AF_INET6
 			if (inet_pton(AF_INET6, ip + strlen("IPv6:"), 
@@ -587,10 +642,6 @@ spf_check_self(ad, as, ap, priv)
 				exit(EX_SOFTWARE);
 			}
 		}
-#else /* USE_POSTFIX */
-		mg_log(LOG_ERR, "spf self used without localaddr specified");
-		return 0;
-#endif /* USE_POSTFIX */
 		break;
 	}
 
